use strict;
use warnings;

sub update_res {

    # Purpose: update the resolutions used by programs PO2R, PSF, and
    # P3DR based on the FSC curve and values of adjustment factors. If
    # $res_flag is non-zero, shift toward higher resolutions.
    #
    # Input arguments:
    #   $po2r_ref   - reference to hash of PO2R input parameters
    #   $p3dr_ref   - reference to hash of P3DR input parameters
    #   $psf_ref    - reference to hash of PSF input parameters
    #   $ppft_ref   - reference to hash of PPFT input parameters
    #   $auto_ref   - reference to hash of control parameters
    #   $res_locut  - resolution where FSC curve crosses fsc_locut
    #   $res_flag   - set to non-zero value if FSC curve did not 
    #                 drop below fsc_locut
    #
    # Return values:
    #   None

    my ($po2r_ref, $p3dr_ref, $psf_ref, $ppft_ref, 
	$auto_ref, $res_locut, $res_flag) = @_;

    my $date = localtime();
    print OUT "Updating resolutions\n";
    print OUT "   Date = $date\n";

    # Calculate resolutions
    # PO2R res_max > P3DR res_min > P3DR res_max
    # PPFT resolution_high = PO2R res_max
    # PSF res_max = P3DR res_min

    # Calculate PO2R res_max
    if($res_flag) {
	$$po2r_ref{res_max} = 1/(1/$res_locut+$$auto_ref{res_adj});
    } else {
	$$po2r_ref{res_max} = $res_locut;
    }
    $$po2r_ref{res_max} = sprintf("%.2f", $$po2r_ref{res_max});

    # Calculate P3DR res_min
    $$p3dr_ref{res_min} = 1/(1/$$po2r_ref{res_max}+$$auto_ref{res_adj});
    $$p3dr_ref{res_min} = sprintf("%.2f", $$p3dr_ref{res_min});

    # Calculate P3DR res_max
    $$p3dr_ref{res_max} = 1/(1/$$p3dr_ref{res_min}+$$auto_ref{gauss_adj});
    $$p3dr_ref{res_max} = sprintf("%.2f", $$p3dr_ref{res_max});

    # Set PPFT resolution_high and PSF res_max
    $$ppft_ref{resolution_high} = $$po2r_ref{res_max};
    $$ppft_ref{resolution_high} = sprintf("%.2f", $$ppft_ref{resolution_high});
    $$psf_ref{res_max} = $$p3dr_ref{res_min};
    $$psf_ref{res_max} = sprintf("%.2f", $$psf_ref{res_max});

    print OUT "   PO2R  res_max        now $$po2r_ref{res_max}\n";
    print OUT "   PPFT resolution_high now $$ppft_ref{resolution_high}\n";
    print OUT "   P3DR res_min         now $$p3dr_ref{res_min}\n";
    print OUT "   P3DR res_max         now $$p3dr_ref{res_max}\n";
    print OUT "   PSF  res_max         now $$psf_ref{res_max}\n";

    print OUT "   (PO2R res_max, PPFT resolution_high)";
    print OUT " set to $$ppft_ref{resolution_high}\n";
    print OUT "   (P3DR res_min, PSF res_max)         ";
    print OUT " set to $$psf_ref{res_max}\n";
    print OUT "   (P3DR res_max)                      ";
    print OUT " set to $$p3dr_ref{res_max}\n";

    print OUT "\n";

    return;
}
1;
